@@ -74,6 +74,55 @@ enum {
 };
 
 
+static void
+ui_component_set_sensitive (BonoboUIComponent *ui,
+			    const char *path,
+			    gboolean sensitive)
+{
+	bonobo_ui_component_set_prop (ui, path, "sensitive",
+				      sensitive ? "1" : "0", NULL);
+}
+
+static void
+gpdf_control_enable_ui (GPdfControl *control, gboolean sensitive)
+{
+	BonoboUIComponent *uic;
+	GPdfView *gpdf_view;
+	gboolean sens_go_back, sens_go_fwd, sens_zoom;
+
+	if (!control->priv->activated)
+		return;
+
+	uic = bonobo_control_get_ui_component (BONOBO_CONTROL (control));
+
+	bonobo_ui_component_freeze (uic, NULL);
+
+	ui_component_set_sensitive (uic, "/commands/FileProperties", sensitive);
+
+	gpdf_view = GPDF_VIEW (control->priv->gpdf_view);
+	ui_component_set_sensitive (
+		uic, "/commands/GoHistoryPrev",
+		sensitive && !gpdf_view_is_first_history (gpdf_view));
+	ui_component_set_sensitive (
+		uic, "/commands/GoHistoryNext",
+		sensitive && !gpdf_view_is_last_history (gpdf_view));
+
+	sens_go_fwd = (sensitive && !gpdf_view_is_last_page (gpdf_view));
+	sens_go_back = (sensitive && !gpdf_view_is_first_page (gpdf_view));
+	ui_component_set_sensitive (uic, "/commands/GoPagePrev", sens_go_back);
+	ui_component_set_sensitive (uic, "/commands/GoPageNext", sens_go_fwd);
+	ui_component_set_sensitive (uic, "/commands/GoPageFirst", sens_go_back);
+	ui_component_set_sensitive (uic, "/commands/GoPageLast", sens_go_fwd);
+
+	sens_zoom = sensitive && !control->priv->has_zoomable_frame;
+	ui_component_set_sensitive (uic, "/commands/ZoomIn", sens_zoom);
+	ui_component_set_sensitive (uic, "/commands/ZoomOut", sens_zoom);
+	ui_component_set_sensitive (uic, "/commands/ZoomFit", sens_zoom);
+	ui_component_set_sensitive (uic, "/commands/ZoomFitWidth", sensitive);
+
+	bonobo_ui_component_thaw (uic, NULL);	
+}
+
 static void
 gpdf_control_properties_dialog_response_cb (GtkDialog *dialog, gint response_id,
 					    gpointer data)
@@ -172,91 +221,13 @@ verb_GoPageLast_cb (BonoboUIComponent *uic, gpointer user_data, const char*cname
 		GPDF_VIEW (GPDF_CONTROL (user_data)->priv->gpdf_view));
 }
 
-static void
-ui_component_set_sensitive (BonoboUIComponent *ui,
-			    const char *path,
-			    gboolean sensitive)
-{
-	bonobo_ui_component_set_prop (ui, path, "sensitive",
-				      sensitive ? "1" : "0", NULL);
-}
-
-static void
-gc_enable_go_back_verbs (GPdfControl *control, gboolean sensitive)
-{
-	BonoboUIComponent *ui;
-
-	g_return_if_fail (GPDF_IS_NON_NULL_CONTROL (control));
-
-	ui = bonobo_control_get_ui_component (BONOBO_CONTROL (control));
-	if (ui == NULL)
-		return;
-
-	ui_component_set_sensitive (ui, "/commands/GoPageFirst", sensitive);
-	ui_component_set_sensitive (ui, "/commands/GoPagePrev", sensitive);
-}
-
-static void
-gc_enable_go_forward_verbs (GPdfControl *control, gboolean sensitive)
-{
-	BonoboUIComponent *ui;
-
-	g_return_if_fail (GPDF_IS_NON_NULL_CONTROL (control));
-
-	ui = bonobo_control_get_ui_component (BONOBO_CONTROL (control));
-	if (ui == NULL)
-		return;
-
-	ui_component_set_sensitive (ui, "/commands/GoPageNext", sensitive);
-	ui_component_set_sensitive (ui, "/commands/GoPageLast", sensitive);
-}
-
-static void
-gc_enable_history_back_verbs (GPdfControl *control, gboolean sensitive)
-{
-	BonoboUIComponent *ui;
-
-	g_return_if_fail (GPDF_IS_NON_NULL_CONTROL (control));
-
-	ui = bonobo_control_get_ui_component (BONOBO_CONTROL (control));
-	if (ui == NULL)
-		return;
-
-	ui_component_set_sensitive (ui, "/commands/GoHistoryPrev", sensitive);
-}
-
-static void
-gc_enable_history_forward_verbs (GPdfControl *control, gboolean sensitive)
-{
-	BonoboUIComponent *ui;
-
-	g_return_if_fail (GPDF_IS_NON_NULL_CONTROL (control));
-
-	ui = bonobo_control_get_ui_component (BONOBO_CONTROL (control));
-	if (ui == NULL)
-		return;
-
-	ui_component_set_sensitive (ui, "/commands/GoHistoryNext", sensitive);
-}
-
 static void
 gc_page_changed_enable_page_buttons (GPdfView *gpdf_view, gint page,
 				     GPdfControl *control)
 {
 	g_return_if_fail (GPDF_IS_NON_NULL_CONTROL (control));
 
-	if (!control->priv->activated)
-		return;
-
-	gpdf_view = GPDF_VIEW (control->priv->gpdf_view);
-
-	gc_enable_go_back_verbs (control, !gpdf_view_is_first_page (gpdf_view));
-	gc_enable_go_forward_verbs (control,
-				    !gpdf_view_is_last_page (gpdf_view));
-	gc_enable_history_back_verbs (control,
-				      !gpdf_view_is_first_history (gpdf_view));
-	gc_enable_history_forward_verbs (control,
-				    !gpdf_view_is_last_history (gpdf_view));
+	gpdf_control_enable_ui (control, TRUE);
 }
 
 
@@ -673,11 +644,7 @@ gpdf_control_activate (BonoboControl *control, gboolean activate)
 
 	gpdf_control->priv->activated = activate;
 
-	/* FIXME Ugly fix to setup controls enable status */
-	gc_page_changed_enable_page_buttons (
-		GPDF_VIEW (gpdf_control->priv->gpdf_view),
-		1,
-		gpdf_control);
+	gpdf_control_enable_ui (gpdf_control, gpdf_control->priv->pdf_doc != NULL);
 
 	BONOBO_CALL_PARENT (
 		BONOBO_CONTROL_CLASS, activate, (control, activate));
@@ -697,9 +664,11 @@ gpdf_control_pdf_doc_changed (GPdfControl *gpdf_control)
 		bonobo_control_get_widget (priv->bonobo_page_control));
 	if (priv->pdf_doc) {
 		gpdf_view_set_pdf_doc (gpdf_view, priv->pdf_doc);
+		gpdf_control_enable_ui (gpdf_control, TRUE);
 		gpdf_page_control_set_total_pages (
 			page_control, priv->pdf_doc->getNumPages ());
 	} else {
+		gpdf_control_enable_ui (gpdf_control, FALSE);
 		gpdf_page_control_set_total_pages (page_control, -1);
 	}
 
