@@ -11,7 +11,6 @@
 #ifndef GPDF_CONTROL_H
 #define GPDF_CONTROL_H
 
-#include "gpdf-persist-stream.h"
 #include "gpdf-g-switch.h"
 #  include <bonobo/bonobo-control.h>
 #include "gpdf-g-switch.h"
@@ -39,9 +38,6 @@ struct _GPdfControlClass {
 };
 
 GType        gpdf_control_get_type  (void);
-GPdfControl *gpdf_control_new       (GPdfPersistStream *gpdf_persist_stream);
-GPdfControl *gpdf_control_construct (GPdfControl *gpdf_control,
-				     GPdfPersistStream *gpdf_persist_stream);
 
 G_END_DECLS
 
